package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.utils.SpringUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.utils.bean.BeanValidators;
import com.ruoyi.common.core.utils.uuid.UUID;
import com.ruoyi.common.datascope.annotation.DataScope;
import com.ruoyi.common.security.service.TokenService;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.domain.SysPost;
import com.ruoyi.system.domain.SysUserPost;
import com.ruoyi.system.domain.SysUserRole;
import com.ruoyi.system.enums.BussLogEnum;
import com.ruoyi.system.mapper.SysPostMapper;
import com.ruoyi.system.mapper.SysRoleMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.mapper.SysUserPostMapper;
import com.ruoyi.system.mapper.SysUserRoleMapper;
import com.ruoyi.system.service.ISysBussLogService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysUserService;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;
import javax.validation.Validator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

@Service
public class SysUserServiceImpl implements ISysUserService {
   private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);
   @Autowired
   private SysUserMapper userMapper;
   @Autowired
   private SysRoleMapper roleMapper;
   @Autowired
   private SysPostMapper postMapper;
   @Autowired
   private SysUserRoleMapper userRoleMapper;
   @Autowired
   private SysUserPostMapper userPostMapper;
   @Autowired
   private ISysConfigService configService;
   @Autowired
   private ISysDeptService deptService;
   @Autowired
   protected Validator validator;
   @Autowired
   private TokenService tokenService;
   @Autowired
   private ISysBussLogService sysBussLogService;

   @DataScope(
      deptAlias = "d",
      userAlias = "u"
   )
   public List<SysUser> selectUserList(SysUser user) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (loginUser.getRoles().contains("dealer")) {
         user.setDealerId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("agent")) {
         user.setAgentId(loginUser.getUserid());
      }

      return this.userMapper.selectUserList(user);
   }

   public List<SysUser> selectList(SysUser user) {
      LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
      if (loginUser.getRoles().contains("dealer")) {
         user.setDealerId(loginUser.getUserid());
      }

      if (loginUser.getRoles().contains("agent")) {
         user.setAgentId(loginUser.getUserid());
      }

      return this.userMapper.selectList(user);
   }

   @DataScope(
      deptAlias = "d",
      userAlias = "u"
   )
   public List<SysUser> selectAllocatedList(SysUser user) {
      return this.userMapper.selectAllocatedList(user);
   }

   @DataScope(
      deptAlias = "d",
      userAlias = "u"
   )
   public List<SysUser> selectUnallocatedList(SysUser user) {
      return this.userMapper.selectUnallocatedList(user);
   }

   public SysUser selectUserByUserName(String userName) {
      return this.userMapper.selectUserByUserName(userName);
   }

   public SysUser selectUserById(Long userId) {
      return this.userMapper.selectUserById(userId);
   }

   public String selectUserRoleGroup(String userName) {
      List<SysRole> list = this.roleMapper.selectRolesByUserName(userName);
      return CollectionUtils.isEmpty(list) ? "" : (String)list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
   }

   public String selectUserPostGroup(String userName) {
      List<SysPost> list = this.postMapper.selectPostsByUserName(userName);
      return CollectionUtils.isEmpty(list) ? "" : (String)list.stream().map(SysPost::getPostName).collect(Collectors.joining(","));
   }

   public boolean checkUserNameUnique(SysUser user) {
      Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
      SysUser info = this.userMapper.checkUserNameUnique(user.getUserName());
      return !StringUtils.isNotNull(info) || info.getUserId() == userId;
   }

   public boolean checkPhoneUnique(SysUser user) {
      Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
      SysUser info = this.userMapper.checkPhoneUnique(user.getPhonenumber());
      return !StringUtils.isNotNull(info) || info.getUserId() == userId;
   }

   public boolean checkEmailUnique(SysUser user) {
      Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
      SysUser info = this.userMapper.checkEmailUnique(user.getEmail());
      return !StringUtils.isNotNull(info) || info.getUserId() == userId;
   }

   public void checkUserAllowed(SysUser user) {
      if (StringUtils.isNotNull(user.getUserId()) && user.isAdmin()) {
         throw new ServiceException("不允许操作超级管理员用户");
      }
   }

   public void checkUserDataScope(Long userId) {
      if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
         SysUser user = new SysUser();
         user.setUserId(userId);
         List<SysUser> users = ((SysUserServiceImpl)SpringUtils.getAopProxy(this)).selectUserList(user);
         if (StringUtils.isEmpty(users)) {
            throw new ServiceException("没有权限访问用户数据！");
         }
      }

   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int insertUser(SysUser user) {
      user.setLicense(UUID.randomUUID().toString().replaceAll("-", ""));
      int rows = this.userMapper.insertUser(user);
      this.insertUserPost(user);
      this.insertUserRole(user);
      return rows;
   }

   public boolean registerUser(SysUser user) {
      return this.userMapper.insertUser(user) > 0;
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int updateUser(SysUser user) {
      Long userId = user.getUserId();
      this.userRoleMapper.deleteUserRoleByUserId(userId);
      this.insertUserRole(user);
      this.userPostMapper.deleteUserPostByUserId(userId);
      this.insertUserPost(user);
      return this.userMapper.updateUser(user);
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public void insertUserAuth(Long userId, Long[] roleIds) {
      this.userRoleMapper.deleteUserRoleByUserId(userId);
      this.insertUserRole(userId, roleIds);
   }

   public int updateUserStatus(SysUser user) {
      return this.userMapper.updateUser(user);
   }

   public int updateUserProfile(SysUser user) {
      return this.userMapper.updateUser(user);
   }

   public boolean updateUserAvatar(String userName, String avatar) {
      return this.userMapper.updateUserAvatar(userName, avatar) > 0;
   }

   public int resetPwd(SysUser user) {
      this.sysBussLogService.insertSysBussLog(BussLogEnum.EditPassword.getType(), user.getUserId(), user.getUserName(), "重置密码为" + user.getPassword());
      user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
      return this.userMapper.updateUser(user);
   }

   public int resetUserPwd(String userName, String password) {
      return this.userMapper.resetUserPwd(userName, password);
   }

   public void insertUserRole(SysUser user) {
      this.insertUserRole(user.getUserId(), user.getRoleIds());
   }

   public void insertUserPost(SysUser user) {
      Long[] posts = user.getPostIds();
      if (StringUtils.isNotEmpty(posts)) {
         List<SysUserPost> list = new ArrayList();
         Long[] var4 = posts;
         int var5 = posts.length;

         for(int var6 = 0; var6 < var5; ++var6) {
            Long postId = var4[var6];
            SysUserPost up = new SysUserPost();
            up.setUserId(user.getUserId());
            up.setPostId(postId);
            list.add(up);
         }

         this.userPostMapper.batchUserPost(list);
      }

   }

   public void insertUserRole(Long userId, Long[] roleIds) {
      if (StringUtils.isNotEmpty(roleIds)) {
         List<SysUserRole> list = new ArrayList();
         Long[] var4 = roleIds;
         int var5 = roleIds.length;

         for(int var6 = 0; var6 < var5; ++var6) {
            Long roleId = var4[var6];
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
         }

         this.userRoleMapper.batchUserRole(list);
      }

   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int deleteUserById(Long userId) {
      this.userRoleMapper.deleteUserRoleByUserId(userId);
      this.userPostMapper.deleteUserPostByUserId(userId);
      return this.userMapper.deleteUserById(userId);
   }

   @Transactional(
      rollbackFor = {Exception.class}
   )
   public int deleteUserByIds(Long[] userIds) {
      Long[] var2 = userIds;
      int var3 = userIds.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Long userId = var2[var4];
         this.checkUserAllowed(new SysUser(userId));
         this.checkUserDataScope(userId);
      }

      this.userRoleMapper.deleteUserRole(userIds);
      this.userPostMapper.deleteUserPost(userIds);
      return this.userMapper.deleteUserByIds(userIds);
   }

   public String importUser(List<SysUser> userList, Boolean isUpdateSupport, String operName) {
      if (!StringUtils.isNull(userList) && userList.size() != 0) {
         int successNum = 0;
         int failureNum = 0;
         StringBuilder successMsg = new StringBuilder();
         StringBuilder failureMsg = new StringBuilder();
         Iterator var8 = userList.iterator();

         while(var8.hasNext()) {
            SysUser user = (SysUser)var8.next();

            String password;
            try {
               SysUser u = this.userMapper.selectUserByUserName(user.getUserName());
               if (StringUtils.isNull(u)) {
                  BeanValidators.validateWithException(this.validator, user, new Class[0]);
                  this.deptService.checkDeptDataScope(user.getDeptId());
                  password = this.configService.selectConfigByKey("sys.user.initPassword");
                  user.setPassword(SecurityUtils.encryptPassword(password));
                  user.setCreateBy(operName);
                  this.userMapper.insertUser(user);
                  ++successNum;
                  successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 导入成功");
               } else if (isUpdateSupport) {
                  BeanValidators.validateWithException(this.validator, user, new Class[0]);
                  this.checkUserAllowed(u);
                  this.checkUserDataScope(u.getUserId());
                  this.deptService.checkDeptDataScope(user.getDeptId());
                  user.setUserId(u.getUserId());
                  user.setUpdateBy(operName);
                  this.userMapper.updateUser(user);
                  ++successNum;
                  successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 更新成功");
               } else {
                  ++failureNum;
                  failureMsg.append("<br/>" + failureNum + "、账号 " + user.getUserName() + " 已存在");
               }
            } catch (Exception var12) {
               Exception e = var12;
               ++failureNum;
               password = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
               failureMsg.append(password + e.getMessage());
               log.error(password, e);
            }
         }

         if (failureNum > 0) {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
         } else {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
            return successMsg.toString();
         }
      } else {
         throw new ServiceException("导入用户数据不能为空！");
      }
   }

   public void updateUserCloakId(Long dealerId, String cloakId) {
      this.userMapper.updateUserCloakId(dealerId, cloakId);
   }
}
